#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 1A.1 - Liste, tuple, ensemble, dictionnaire, liste chaînée, coût des opérations
# 
# Exemples de containers, list, tuple, set, dict.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# Python propose différents [containers](https://docs.python.org/3.4/tutorial/datastructures.html#) pour stocker des éléments. Voici les plus courants :
# 
# - [list](https://docs.python.org/3.4/tutorial/datastructures.html#more-on-lists) : tableau d'éléments indexés de 0 à $n$ exclu auquel on peut ajouter ou retirer des éléments
# - [dict](https://docs.python.org/3.4/tutorial/datastructures.html#dictionaries) : tableau d'éléments indexés par des types immuables auquel on peut ajouter ou retirer des éléments
# - [tuple](https://docs.python.org/3.4/tutorial/datastructures.html#tuples-and-sequences) : tableau d'éléments indexés de 0 à $n$ exclu qu'on ne peut pas modifier
# - [set](https://docs.python.org/3.4/tutorial/datastructures.html#sets) : tableau d'éléments uniques non indexés
# - [frozenset](https://docs.python.org/3.4/tutorial/datastructures.html#sets) : ``set`` immuables (non modifiable)
# - [deque](https://docs.python.org/3.4/library/collections.html#collections.deque) : presque équivalent à une listes, la différent vient de l'implémentation, les mêmes opérations n'auront pas les mêmes coûts (deque = [liste chaînée](http://fr.wikipedia.org/wiki/Liste_cha%C3%AEn%C3%A9e))
# 
# D'autres containers sont disponibles via le module [collections](https://docs.python.org/3.4/library/collections.html). Tous proposent de stocker un nombre variables d'éléments. Deux aspects difféèrent :
# 
# - la façon de désigner un élément de l'ensemble
# - le coût de certaines opérations, il faut choisir qui minisera le coût des opérations pour votre programme

# ### Insertion avec ``list`` et ``deque``
# 
# On veut comparer les coûts d'insertion en début et fin de liste pour un grand nombre d'éléments.

# In[2]:


import time, collections
N = 1000000

for p in range(0,3):
    print("passage ", p)
    print("  insertion en fin")

    li = list()
    a = time.perf_counter()
    for i in range(0,N) :
        li.append(i)
    b = time.perf_counter()
    print("    list", N, "éléments, temps par éléments :", (b-a)/N)    
        
    li = collections.deque()
    a = time.perf_counter()
    for i in range(0,N) :
        li.append(i)
    b = time.perf_counter()
    print("    deque", N, "éléments, temps par éléments :", (b-a)/N)    
    
    print("  insertion au début")
    li = collections.deque()
    a = time.perf_counter()
    for i in range(0,N) :
        li.appendleft(i)
    b = time.perf_counter()
    print("    deque", N, "éléments, temps par éléments :", (b-a)/N)    
    
    N2 = N // 100
    li = list()
    a = time.perf_counter()
    for i in range(0,N2) :
        li.insert(0,i)
    b = time.perf_counter()
    print("    list", N, "éléments, temps par éléments :", (b-a)/N)   


# On voit que l'insertion au début du tableau est beaucoup plus coûteuse pour une liste que pour un ``deque``.

# ### Un élément dans un ensemble
# 
# Faut-il écrire ``i in [0,1]`` ou ``i in (0,1)`` ou ... Essayons.

# In[3]:


import time, collections
N = 100000
lens = list(range(0,1000))
tens = tuple(lens)
sens = set(lens)
fens = frozenset(lens)

for p in range(0,3):
    print("passage",p)
    a = time.perf_counter()
    s = 0
    for i in range(0,N) :
        if i in lens : s += 1
    b = time.perf_counter()
    print("  list", N, "fois, temps par éléments :", (b-a)/N)    
    
    a = time.perf_counter()
    s = 0
    for i in range(0,N) :
        if i in tens : s += 1
    b = time.perf_counter()
    print("  tuple", N, "fois, temps par éléments :", (b-a)/N)    
    
    a = time.perf_counter()
    s = 0
    for i in range(0,N) :
        if i in sens : s += 1
    b = time.perf_counter()
    print("  set", N, "fois, temps par éléments :", (b-a)/N)    
    
    a = time.perf_counter()
    s = 0
    for i in range(0,N) :
        if i in fens : s += 1
    b = time.perf_counter()
    print("  frozenset", N, "fois, temps par éléments :", (b-a)/N)    


# Il apparaît que les ensemble ``set`` ou ``frozenset`` sont beaucoup plus rapides. Plus l'ensemble est grand, plus cette différence est importante.

# In[4]: